@@ -1,12 +1,16 @@
-"""JSON encoding functions using EAK-Rules."""
+"""JSON encoding functions using PEAK-Rules."""
 
-import datetime
-import decimal
+from datetime import date, datetime
+from decimal import Decimal
 
-from peak.rules import abstract
-from prioritized_methods import prioritized_when, prioritized_around
+from peak.rules import abstract, around, when
 
-from simplejson import JSONEncoder
+try:
+    from json import JSONEncoder
+except ImportError: # Python < 2.6
+    from simplejson import JSONEncoder
+
+__all__ = ['jsonify', 'encode', 'encode_iter']
 
 
 # Global options
@@ -16,7 +20,6 @@ descent_bases = True
 
 # Specific encoding functions
 
-@abstract()
 def jsonify(obj):
     """Generic function for converting objects to JSON.
 
@@ -25,29 +28,53 @@ def jsonify(obj):
     (with string keys), and strings, ints, and floats.
 
     """
-    raise NotImplementedError
+    return jsonify_default(obj)
 
 # This is for easier usage and backward compatibility:
-jsonify.when = prioritized_when.__get__(jsonify)
-jsonify.around = prioritized_around.__get__(jsonify)
+jsonify.when = when.__get__(jsonify)
+jsonify.around = around.__get__(jsonify)
 
 
-@jsonify.when("isinstance(obj, (datetime.date, datetime.datetime))", prio=-1)
-def jsonify_datetime(obj):
-    """JSONify datetime and date objects."""
-    return str(obj)
+# Explicit encoding function
 
-@jsonify.when("isinstance(obj, decimal.Decimal)", prio=-1)
-def jsonify_decimal(obj):
-    """JSONify decimal objects."""
-    return float(obj)
-
-@jsonify.when("hasattr(obj, '__json__')", prio=-1)
+@jsonify.around("hasattr(obj, '__json__')")
 def jsonify_explicit(obj):
     """JSONify objects with explicit JSONification method."""
     return obj.__json__()
 
 
+# Default encoding functions
+
+@abstract
+def jsonify_default(obj):
+    """Default JSONify method for Python objects."""
+    return NotImplementedError
+
+jsonify_default.when = when.__get__(jsonify_default)
+jsonify_default.around = around.__get__(jsonify_default)
+
+@jsonify_default.when("obj is None"
+    " or isinstance(obj, (basestring, bool, int, long, float, list, dict))")
+def jsonify_serializable(obj):
+    """JSONify simple serializable objects."""
+    return obj
+
+@jsonify_default.when("isinstance(obj, (set, frozenset, tuple))")
+def jsonify_list_like(obj):
+    """JSONify simple list-like objects."""
+    return list(obj)
+
+@jsonify_default.when("isinstance(obj, Decimal)")
+def jsonify_decimal(obj):
+    """JSONify Decimal objects."""
+    return float(obj)
+
+@jsonify.when("isinstance(obj, (date, datetime))")
+def jsonify_datetime(obj):
+    """JSONify date and datetime objects."""
+    return str(obj)
+
+
 # SQLObject support
 
 try:
@@ -71,7 +98,7 @@ try:
         return (isinstance(obj, SQLObject)
             and hasattr(obj.__class__, 'sqlmeta'))
 
-    @jsonify.when("is_sqlobject(obj) and not hasattr(obj, '__json__')", prio=-1)
+    @jsonify_default.when("is_sqlobject(obj)")
     def jsonify_sqlobject(obj):
         """JSONify SQLObjects."""
         result = {'id': obj.id}
@@ -85,7 +112,7 @@ try:
         pass
     else:
 
-        @jsonify.when("isinstance(obj, SelectResultsClass)", prio=-1)
+        @jsonify_default.when("isinstance(obj, SelectResultsClass)")
         def jsonify_select_results(obj):
             """JSONify SQLObject.SelectResults."""
             return list(obj)
@@ -107,8 +134,7 @@ try:
         def is_saobject(obj):
             return hasattr(obj, '_sa_class_manager')
 
-        @jsonify.when("is_saobject(obj) and not hasattr(obj, '__json__')",
-                      prio=-1)
+        @jsonify_default.when("is_saobject(obj)")
         def jsonify_saobject(obj):
             """JSONify SQLAlchemy objects."""
             props = {}
@@ -123,8 +149,7 @@ try:
             return (hasattr(obj, 'c')
                 and isinstance(obj.c, OrderedProperties))
 
-        @jsonify.when("is_saobject(obj) and not hasattr(obj, '__json__')",
-                      prio=-1)
+        @jsonify_default.when("is_saobject(obj)")
         def jsonify_saobject(obj):
             """JSONify SQLAlchemy objects."""
             props = {}
@@ -139,16 +164,26 @@ try:
 
         else: # SQLAlchemy < 0.4
 
-            @jsonify.when("isinstance(obj, InstrumentedList)", prio=-1)
+            @jsonify_default.when("isinstance(obj, InstrumentedList)")
             def jsonify_instrumented_list(obj):
                 """JSONify SQLAlchemy instrumented lists."""
                 return list(obj)
 
+    from sqlalchemy.engine.base import ResultProxy, RowProxy
+
+    @jsonify_default.when("isinstance(obj, ResultProxy)")
+    def jsonify_saproxy(obj):
+        return list(obj)
+
+    @jsonify_default.when("isinstance(obj, RowProxy)")
+    def jsonify_saproxy(obj):
+        return dict(obj)
+
 except ImportError:
     pass
 
 
-# JSON Encoder class
+# JSON encoder class
 
 class GenericJSON(JSONEncoder):
 
